import sys
from os import path

__author__ = 'Magnus "Loxosceles" Henkel'
__email__ = 'loxosceles@gmx.de'
__version__ = '0.1.0'

ROOT_DIR = path.dirname(path.dirname(path.abspath(__file__)))
sys.path.insert(0, ROOT_DIR)




#!/usr/bin/env python3

"""
Tests for `dot_configs` module.
"""

import pytest
import json
from os import path
from dot_configs.dot_configs import (Configurations, AttributeWrapper)

DATA_JSON_DIR = pytest.DATA_ROOT

@pytest.fixture
def attributewrapper():
    return AttributeWrapper('test')

@pytest.fixture
@pytest.mark.datafiles(DATA_JSON_DIR)
def cfg(datafiles):
    return path.join(str(datafiles), 'conf.json')

@pytest.fixture
@pytest.mark.datafiles(DATA_JSON_DIR)
def cfg_bad(datafiles):
    return path.join(str(datafiles), 'bad.json')

@pytest.fixture
@pytest.mark.datafiles(DATA_JSON_DIR)
def config_dict(cfg):
    with open(cfg, 'r') as infile:
        d = json.load(infile)
    return d

@pytest.fixture
@pytest.mark.datafiles(DATA_JSON_DIR)
def configs(cfg):
    c = Configurations(cfg)
    c = c.get_configuration()
    return c

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_validate_flag_is_true_when_parsing_correctly(cfg):
    c = Configurations(cfg)
    assert c.json_validated is True

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_validate_flag_is_false_when_parsing_incorrectly(cfg_bad):
    c = Configurations(cfg_bad)
    assert c.json_validated is False

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_set_new_node(configs):
    configs.set('args')
    assert isinstance(configs.args, AttributeWrapper)

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_set_new_node_and_value(configs):
    configs.architecture.set('args', 'test')
    assert configs.architecture.args == 'test'

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_deep_get_node_and_value(configs):
    epochs = configs.deep_get(['architecture', 'cnn', 'training_params', 'epochs'])
    assert epochs == 4

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_deep_get_node_only_returns_dict_object(configs, config_dict):
    tp = configs.deep_get(['architecture', 'cnn', 'training_params'])
    assert tp == config_dict["architecture"]["cnn"]["training_params"]

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_int_type_on_attributewrapper(configs):
    assert configs.architecture.cnn.training_params.epochs == 4

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_list_type_on_attributewrapper(configs):
    assert configs.spectrograms.frame_sizes == [1024, 2048, 4096]

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_str_type_on_attributewrapper(configs):
    assert configs.paths.dirs.store == '/path/to/store'

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_float_type_on_attributewrapper(configs):
    assert configs.training_data.test_size == 0.10

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_bool_type_on_attributewrapper(configs):
    assert configs.architecture.cnn.audio_params.filterbank == True

@pytest.mark.skip(reason="Not implemented yet.")
@pytest.mark.datafiles(DATA_JSON_DIR)
def test_none_type_on_attributewrapper(configs):
    assert configs.args is None

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_IntWrapper_behaves_like_int(attributewrapper, configs):
    IntWrapper = attributewrapper.wrappers["int"]
    number = IntWrapper(1)
    assert (number + number) == 2

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_FloatWrapper_behaves_like_float(attributewrapper, configs):
    FloatWrapper = attributewrapper.wrappers["float"]
    number = FloatWrapper(1.2)
    assert (number + number) == 2.4

@pytest.mark.skip(reason="Not implemented yet")
@pytest.mark.datafiles(DATA_JSON_DIR)
def test_NoneWrapper_is_None(attributewrapper, configs):
    NoneWrapper = attributewrapper.wrappers["NoneType"]
    none = NoneWrapper("Will result in 'None' anyways")
    assert str(none) == str(None)

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_StrWrapper_behaves_like_str(attributewrapper, configs):
    StrWrapper = attributewrapper.wrappers["str"]
    string = StrWrapper('string')
    assert (string + string) == "stringstring"

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_leaf_on_attributewrapper_is_correct_type(attributewrapper, configs):
    epochs_class = configs.architecture.cnn.training_params.epochs.__class__
    assert isinstance(epochs_class, attributewrapper.wrappers["int"].__class__)

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_attributewrapper_stores_internal_dict_correctly(configs, config_dict):
    assert configs.paths._store == config_dict["paths"]

@pytest.mark.datafiles(DATA_JSON_DIR)
def test_root_attributewrapper_stores_internal_dict_correctly(configs, config_dict):
    assert configs._store == config_dict



#!/usr/bin/env python3

# Imports
from os import path
import json
from collections import MutableMapping


class NoneWrapper:
    def __init__(self, value):
        self.value = None

    def __name__(self):
        return str(None)

    def __str__(self):
        return str(None)

    def __repr__(self):
        return str(None)

class IntWrapper(int):
    def __init__(self, value):
        super().__init__()

class FloatWrapper(float):
    def __init__(self, value):
        super().__init__()

class StrWrapper(str):
    def __init__(self, value):
        super().__init__()

class ListWrapper(list):
    def __init__(self, value):
        super().__init__(value)

class TupleWrapper(tuple):
    def __init__(self, value):
        super().__init__()

class BoolWrapper(int):
    def __init__(self, value):
        super().__init__()


class AttributeWrapper(MutableMapping):
    """TODO: Docstring for AttributeWrapper."""

    wrappers = {"int": IntWrapper,
                "IntWrapper": IntWrapper,
                "float": FloatWrapper,
                "FloatWrapper": FloatWrapper,
                "str": StrWrapper,
                "StrWrapper": StrWrapper,
                "list": ListWrapper,
                "ListWrapper": ListWrapper,
                "tuple": TupleWrapper,
                "TupleWrapper": TupleWrapper,
                "bool": IntWrapper,
                "BoolWrapper": IntWrapper,
                "NoneType": NoneWrapper
                }

    def __init__(self, key, *args, **kwargs):
        self.key = key
        self._store = {}

    def set(self, key, *args):
        if not args:
            # 1
            aw = AttributeWrapper(key)
            self._store[key] = aw
            setattr(self, aw.key, aw)
            setattr(aw, '_previous', self)
            #  print("1: ", type(aw))
        else:
            # 2
            value = args[0]
            if isinstance(value, dict):
                #  print("3: ", self.key)
                # 3: type AttributeWrapper
                aw = AttributeWrapper(key)
                aw._store = value
                setattr(aw, '_previous', self)
                setattr(self, aw.key, aw)
            else:
                # 4: Str/Float/List/../Wrapper
                #  aw = AttributeWrapper.wrappers[str(type(value))](value)
                aw = AttributeWrapper.wrappers[value.__class__.__name__](value)
                self._store[key] = value
                #  setattr(self, key, type(value)(aw))
                setattr(self, key, aw)
                setattr(aw, '_previous', self)

    def deep_get(self, *args):
        if not args:
            # Nothing in list
            return self
        else:
            l = args[0]
            if len(l) == 1 and not isinstance(l[0], dict):
                return getattr(self, l[0])
            elif len(l) == 1 and isinstance(l[0], dict):
                return l[0]
            elif l:
                try:
                    # if list item is object, take the key
                    l[0] = l[0].key
                except:
                    pass
                try:
                    #  return getattr(self, l[0].key).deep_get(l[1:])
                    return getattr(self, l[0]).deep_get(l[1:])
                except AttributeError as e:
                    # Reached TypeWrapper
                    print(e)
                    #  return getattr(self, l[0]).deep_get(l[1:])
            else:
                # Only AttributeWrappers in list
                return self

    def __getitem__(self, key):
        return self._store[key]

    def __setitem__(self, key, value):
        self._store[key] = value

    def __delitem__(self, key):
        del self._store[key]

    def __iter__(self):
        return iter(self._store)

    def __len__(self):
        return len(self._store)

    def __repr__(self):
        return str(self._store)

class Configurations():
    """Object holding configuration variables."""

    def __init__(self, config_file):
        self.configuration = AttributeWrapper("configurations")
        self.json_validated = False
        cf = path.abspath(config_file)
        configs = self._import_configs(cf)
        self.configuration._store = configs
        self.configuration._previous = None
        self._parse_json(configs)

    @property
    def validated(self):
        return self.json_validated

    def get_configuration(self):
        """docstring for get_configuration"""
        return self.configuration

    def _import_configs(self, conf_file):
        """Import configurations from tf_conf.json."""
        try:
            with open(str(conf_file), 'r') as infile:
                conf = json.load(infile)
            self.json_validated = True
            return conf
        except ValueError as e:
            print("Decoding the JSON config file has failed. Please make sure the format is correct.")
            self.json_validated = False
            #  return None

    def _parse_json(self, sub_tree, prev_node=None, prev_key=None):
        """Parses json object recursively and returns path and value."""

        if isinstance(sub_tree, dict):
            for key, value in sub_tree.items():

                try:
                    curr_node = getattr(prev_node, prev_key)
                except TypeError as e:
                    curr_node = self.configuration

                curr_node.set(key, value)
                self._parse_json(value, curr_node, key)
        else:
            try:
                prev_node.set(prev_key, sub_tree)
            except Exception as e:
                print(e)



#!/usr/bin/env python

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open('README.rst').read()
doclink = """
Documentation
-------------

The full documentation is at http://dot_configs.rtfd.org."""
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='dot_configs',
    version='0.1.1',
    description='Import and handle project configurations as dot-separated configs object',
    long_description=readme + '\n\n' + doclink + '\n\n' + history,
    author='Magnus "Loxosceles" Henkel',
    author_email='loxosceles@gmx.de',
    url='https://gitlab.com/loxosceles/dot_configs',
    packages=[
        'dot_configs',
    ],
    package_dir={'dot_configs': 'dot_configs'},
    include_package_data=True,
    install_requires=[
    ],
    license='GNU GPL v3.0',
    zip_safe=False,
    keywords='dot_configs',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)
